package HTML::FormFu::Element::_MultiText;

use strict;
use base 'HTML::FormFu::Element::_MultiElement', 'HTML::FormFu::Element::Text';

1;

__END__

=head1 NAME

HTML::FormFu::Element::_MultiText

=head1 AUTHOR

Carl Franks, C<cfranks@cpan.org>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.
